/* -*- c -*- */
#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"

#include "dictionary.c"
#include "words.c"

#define MAXDICS 10
Dictionary   *dics[MAXDICS];
WordList     *wls[MAXDICS];
WordLstNode **wln[MAXDICS];
int first_empty_dic=0;
int first_empty_wl=0;
int inited = 0;



void init(void) {
    int i;
    for (i=0;i<MAXDICS;dics[i++]=NULL);
    for (i=0;i<MAXDICS; wls[i++]=NULL);
    for (i=0;i<MAXDICS; wln[i++]=NULL);
    inited = 1;
}

MODULE = NAT		PACKAGE = NAT

U32
dicgetsize(id)
         int id
   INIT:
         U32 size;
   CODE:
         if (id > MAXDICS || id < 0 || !dics[id]) {
	     size = -1;
         } else {
	     size = dictionary_get_size(dics[id]);
	 }
         RETVAL = size;
   OUTPUT:
         RETVAL



U32
dicgetocc(id, word)
         int id
         U32 word
   INIT:
         U32 occ;
   CODE:
         if (id > MAXDICS || id < 0 || !dics[id]) {
	     occ = -1;
         } else {
	     occ = dictionary_get_occ(dics[id], word);
	 }
         RETVAL = occ;
   OUTPUT:
         RETVAL


SV*
dicgetvals(id, word)
         int id
         U32 word
   INIT:
         AV* array;
   CODE:
         if (id > MAXDICS || id < 0 || !dics[id]) {
	     XSRETURN_UNDEF;
         } else {
	     int i;
	     guint32 wid;
	     float val;
	     array = newAV();
	     for (i=0;i<MAXENTRY;i++) {
		 wid = dictionary_get_id(dics[id],word,i);
		 val = dictionary_get_val(dics[id],word,i);
		 if (val == 0.0) break;
		 av_push(array, newSVuv(wid));
		 av_push(array, newSVnv(val));
	     }
	 }
         RETVAL = newRV_noinc((SV*)array);
   OUTPUT:
         RETVAL

void
dicclose(id)
         int id
   CODE:
         if (id > MAXDICS || id < 0) return;
         if (dics[id]) dictionary_free(dics[id]);
         dics[id] = NULL;
         if (id < first_empty_dic) first_empty_dic = id;


int
dicsave(id, filename)
        int id
        char *filename
   CODE:
        if (id > MAXDICS || id < 0 || !dics[id]) {
            RETVAL = 0;
	} else {
	    RETVAL = dictionary_save(dics[id], filename);
	}
   OUTPUT:
        RETVAL


int
dicadd(id1, id2)
        int id1
        int id2
   INIT:
        int nid;
   CODE:
        if (id1 > MAXDICS || id1 < 0 || !dics[id1] ||
	    id2 > MAXDICS || id2 < 0 || !dics[id2] ||
	    first_empty_dic == MAXDICS) {
	    nid = -1;
	} else {
	    nid = first_empty_dic;
	    dics[nid] = dictionary_add(dics[id1], dics[id2]);
            while(first_empty_dic < MAXDICS) {
		if (!dics[first_empty_dic]) break;
		first_empty_dic++;
	    }
	}
        RETVAL = nid;
   OUTPUT:
        RETVAL


int
dicopen(filename)
        char* filename
   INIT:
        int i = first_empty_dic;
   CODE:
        if (!inited) init();
        if (first_empty_dic == MAXDICS) {
	    RETVAL = -1;
        } else {
            dics[first_empty_dic] = dictionary_open(filename);
            while(first_empty_dic < MAXDICS) {
              if (!dics[first_empty_dic]) break;
              first_empty_dic++;
	    }
	RETVAL = i;
   	}

   OUTPUT:
        RETVAL


double
fulldicsim(dic1, dic2, s1, s2)
        int dic1
        int dic2
        AV* s1
        AV* s2
   CODE:
        {
	    double tmp;
	    if (dic1 > MAXDICS || dic1 < 0 || !dics[dic1] ||
		dic2 > MAXDICS || dic2 < 0 || !dics[dic2]) {
		RETVAL = 0;
	    } else {
		U32 s1size, s2size, i;
		int *ss1, *ss2;
		
		s1size = av_len(s1)+1;
		s2size = av_len(s2)+1;
		ss1 = (int*)malloc(sizeof(int)*s1size);
		ss2 = (int*)malloc(sizeof(int)*s2size);
		for(i=0;i<s1size;i++) {
		    SV **sv = av_fetch(s1,i,1);
		    ss1[i] = SvNV(*sv);
		}
		for(i=0;i<s2size;i++) {
		    SV **sv = av_fetch(s2,i,1);
		    ss2[i] = SvNV(*sv);
		}
		tmp = dictionary_sentence_similarity(dics[dic1],
						     ss1, s1size,
						     ss2, s2size);
		tmp += dictionary_sentence_similarity(dics[dic2],
						      ss2, s2size,
						      ss1, s1size);
		RETVAL = tmp/2;
		free(ss1);
		free(ss2);
	    }
	}
    OUTPUT:
	RETVAL


















int
wlopen(filename)
        char* filename
   INIT:
        int i = first_empty_wl;
   CODE:
        if (!inited) init();
        if (first_empty_wl == MAXDICS) {
	    RETVAL = -1;
        } else {
            wls[first_empty_wl] = word_list_load(filename, &wln[first_empty_wl]);
            while(first_empty_wl < MAXDICS) {
              if (!wls[first_empty_wl]) break;
              first_empty_wl++;
	    }
	RETVAL = i;
   	}

   OUTPUT:
        RETVAL


void
wlclose(id)
         int id
   CODE:
         if (id > MAXDICS || id < 0) return;
         if (wls[id]) word_list_free(wls[id]);
         free(wln[id]);
         wls[id] = NULL;
         wln[id] = NULL;
         if (id < first_empty_wl) first_empty_wl = id;


char*
wlgetbyid(id, word)
         int id
         U32 word
   INIT:
         char* str;
   CODE:
         if (id > MAXDICS || id < 0 || !wln[id]) {
	     str = g_strdup("");
         } else {
	     str = word_list_get_by_id(wln[id], word);
	     if (!str) str = g_strdup("(null)");
	 }
         RETVAL = str;
         /*   CLEANUP: free(str);  /* or similar call to free the memory */
   OUTPUT:
         RETVAL




U32
wlgetbyword(id, word)
         int id
         char *word
   INIT:
         U32 wid;
   CODE:
         if (id > MAXDICS || id < 0 || !wln[id]) {
	     wid = 0;
         } else {
	     wid = word_list_get_id(wls[id], word);
	 }
         RETVAL = wid;
   OUTPUT:
         RETVAL
