package NAT::Translator::method1;
use strict;
use Data::Dumper;
use NAT::PCorpus;
use NAT::Dict;



sub method {
  my ($self, $search_string, $size, $source, $target) = @_;

  return process($self, $size, $search_string, $target);
}

sub process {
  my ($self, $size, $search_string, $target) = @_;

  my @target = split/\s+/, $target;
  my $tsize = $#target + 1;
  my @target_ids = @target;

  my $margin = 1;
  my $low = $size-$margin;
  $low = 1 if $low < 1;

  my %data;
  for my $wsize ($low..$size+$margin) {
    my ($best_string, $best_value) = get_best_window($self,$size,$wsize,$search_string,$tsize,\@target_ids);
    $best_string =~ m!(\d+) (\d+)!;
    $best_string = join(" ",map {$self->{tLex}->word_from_id($_) } @target[$1..$2]);
    $data{$best_string} = $best_value;

  }
  my $best_str = (sort {$data{$b}<=>$data{$a} || length($a) > length($b)} keys %data)[0];
  my $best_val = $data{$best_str};

  return ($best_str,$best_val);

}

sub get_best_window {
  my ($self, $sss, $window_size, $search_string, $tsize, $target) = @_;
  my %answers = ();
  for my $i (0..$tsize-$window_size) {
    my @target_ids = @{$target}[$i..$i+$window_size-1];
    my $tstr = "$i ".($i+$window_size-1);
    $answers{$tstr} = NAT::full_sentence_similarity({size => $sss,
						     sourceDict => $self->{sDict},
						     targetDict => $self->{tDict}},
								   $search_string, \@target_ids);
  }
  my $best_string = (sort {$answers{$b}<=>$answers{$a}} keys %answers)[0];
  my $best_value = $answers{$best_string};
  return ($best_string, $best_value);
}

1;
