#!/usr/bin/perl -w

use strict;
use NAT;
use Config::Simple;
use CGI qw/:standard/;

# The idea is to check a configuration file with pairs of aligned
# corpus... that is, for example on each line a pair of files which
# have been aligned with the 'nat-these' tool. This means that if one
# line contains 'Corpus-EN Corpus-PT' then there should be a directory
# named 'Corpus-EN-Corpus-PT' with created files by the nat-these
# tool. More, in fact: we hope there are files like a .db or a .store
# file for each language. These files are tested.

# This variable holds the corpus filenames. It is an array of
# pairs. Each pair element is one of the corpus filename.
my @CORPUS = ();

# First, let's open the configuration file.
#
# At the moment, it should be named 'nat-main.cnf' and, as pointed
# earlier, each line contains two file names separated by spaces.
open CONF, "< nat-main.cnf" or die "Cannot open configuration file nat-main.cnf: $!";
while (<CONF>) {
  chomp;
  push @CORPUS, $_;
}
close CONF;

print header;
print while(<DATA>);

print "<table cellspacing='0'>";
print Tr(th(["Corpus name","Corpus","Dictionary","Description"]));
my $crp;
for $crp (@CORPUS) {
  my $nat = NAT::nat_quick_open($crp);
  print Tr(td([$crp,
	       a({-href=> "nat-search.cgi?path=$crp"},"search"),
	       a({-href=> "nat-browse.cgi?path=$crp"},"navigate")]),
	   td({-style=>"text-align: left"},
	      $nat->{conf}->param("description")
	     ));
}
print "</table>";

__END__

<html>
 <head>
  <title>nat-main</title>
  <style>
   a         { text-decoration: none; color: #009; }
   a:hover   { text-decoration: underline; color: #00f; }
   table { border: solid 1px #000; padding: 0px; }
   th { background-color: #efe;
        border-bottom: solid 1px #000;
	   border-right: solid 1px #000;
	   margin: 0px;
           text-align: center;
	   padding: 2px}
   td    { padding: 2px;
           text-align: center;
           vertical-align: top;
           border-bottom: dotted 1px #000;
	   border-right: dotted 1px #000; }
  </style>
 </head>
 <body>

  <h2>NATools corpus browser</h2>
