/* -*- Mode: C; c-file-style: "stroustrup" -*- */

/**********************************************
 * SEE COPYING FILE FOR COPYRIGHT INFORMATION *
 **********************************************/

#include <stdlib.h>
#include <stdio.h>
#include <zlib.h>
#include <string.h>
#include "dictionary.h"

int cmp(const void* pair1, const void* pair2) {
    return ((DicPair*)pair2)->val > ((DicPair*)pair1)->val;
}

int dictionary_save(Dictionary *dic, const char *name) {
    FILE *gzf;

    gzf = gzopen(name, "wb");
    if (!gzf) return 0;

    if (gzwrite(gzf, &dic->size, sizeof(guint32)) != sizeof(guint32)) return 0;
    if (gzwrite(gzf, dic->pairs, sizeof(DicPair)*MAXENTRY*(dic->size+1)) != 
	sizeof(DicPair)*MAXENTRY*(dic->size+1)) return 0;
    if (gzwrite(gzf, dic->occurs, sizeof(guint32)*(dic->size+1)) != 
	sizeof(guint32)*(dic->size+1)) return 0;
    gzclose(gzf);
    return 1;
}

Dictionary* dictionary_open(const char *name) {
    Dictionary *dic;
    FILE *gzf;
    guint32 size;
    gzf = gzopen(name, "rb");
    if (!gzf) return NULL;
    if (gzread(gzf, &size, sizeof(guint32)) != sizeof(guint32)) return NULL;
    dic = dictionary_new(size);
    if (!dic) return NULL;
    if (gzread(gzf, dic->pairs, sizeof(DicPair)*MAXENTRY*(dic->size+1))	!=
	sizeof(DicPair)*MAXENTRY*(dic->size+1)) {
	dictionary_free(dic);
	return NULL;
    }
    if (gzread(gzf, dic->occurs, sizeof(guint32)*(dic->size+1)) !=
	sizeof(guint32)*(dic->size+1)) {
	dictionary_free(dic);
	return NULL;
    }
    return dic;
}
    
    
guint32     dictionary_get_id(Dictionary* dic, guint32 wid, guint32 offset)  {
    if (wid >= dic->size || offset >= MAXENTRY) return 0;
    return DIC_POS(dic->pairs, wid, offset).id;
}

float       dictionary_get_val(Dictionary* dic, guint32 wid, guint32 offset) {
    if (wid >= dic->size || offset >= MAXENTRY) return 0;
    return DIC_POS(dic->pairs, wid, offset).val;
}

Dictionary *dictionary_set_id(Dictionary* dic, guint32 wid, guint32 offset, guint32 id) {
    DIC_POS(dic->pairs, wid, offset).id = id;
    return dic;
}

Dictionary *dictionary_set_val(Dictionary* dic, guint32 wid, guint32 offset, float val) {
    DIC_POS(dic->pairs, wid, offset).val = val;
    return dic;
}

Dictionary *dictionary_set_occ(Dictionary* dic, guint32 wid, guint32 count) {
    dic->occurs[wid] = count;
    return dic;
}
guint32 dictionary_get_occ(Dictionary* dic, guint32 wid) {
    if (wid >= dic->size) return 0;
    return dic->occurs[wid];
}

guint32     dictionary_get_size(Dictionary* dic) {
    return dic->size;
}

Dictionary *dictionary_new(guint32 size) {
    Dictionary *new;

    new = (Dictionary*)malloc(sizeof(Dictionary));
    if (!new) return new;

    new -> pairs = (DicPair*)malloc(sizeof(DicPair)*MAXENTRY*(size+1));
    if (!new -> pairs) {
	free(new);
	return NULL;
    }
    memset(new -> pairs, 0, sizeof(DicPair)*MAXENTRY*(size+1));

    new -> occurs = (guint32*)malloc(sizeof(guint32)*MAXENTRY*(size+1));
    if (!new -> occurs) {
	free(new -> pairs);
	free(new);
	return NULL;
    }

    memset(new -> occurs, 0, sizeof(guint32)*MAXENTRY*(size+1));
    new -> size = size;

    return new;
}

void dictionary_free(Dictionary *dic) {
    free(dic -> occurs);
    free(dic -> pairs);
    free(dic);
}

Dictionary* dictionary_add(Dictionary *dic1, Dictionary *dic2) {
    Dictionary *new;
    int j,k;
    guint32 size1 = 0, size2 = 0;
    guint32 i, count, id;
    DicPair buffer1[MAXENTRY * 2], buffer2[MAXENTRY * 2];
    guint32 new_size = dic1->size > dic2->size ? dic1->size : dic2->size;

    for (i = 0; i < new_size; i++) {
	size1 +=  dictionary_get_occ(dic1, i);
	size2 +=  dictionary_get_occ(dic2, i);
    }

    new = dictionary_new(new_size);
    for (i = 0; i < new_size; i++) {
	memset(buffer1, 0, sizeof(DicPair)*MAXENTRY*2);
	memset(buffer2, 0, sizeof(DicPair)*MAXENTRY*2);

	for (j = 0; j < MAXENTRY; j++) {
	    buffer1[j].id  = dictionary_get_id(dic1, i, j);
	    buffer1[j].val = dictionary_get_val(dic1, i, j);
	}

	for (j = 0; j < MAXENTRY; j++) {
	    id = dictionary_get_id(dic2, i, j);
	    for (k = 0; k < MAXENTRY; k++) {
		if (id == buffer1[k].id) {
		    buffer2[k].id = id;
		    buffer2[k].val = dictionary_get_val(dic2, i, j);
		    break;
		}
	    }
	    if (k == MAXENTRY) { /* no existe no dic1 */
		for (; k<MAXENTRY*2; k++) {
		    if (buffer2[k].id == 0) {
			buffer2[k].id = id;
			buffer2[k].val = dictionary_get_val(dic2, i, j);
			break;
		    }
		}
	    }
	}

	/* OK. Neste ponto devemos ter os dois buffers prontos pare
	 * serem somados */
	for (j = 0; j< MAXENTRY*2; j++) {
	    float tmp;
	    float v1,v2;

	    float occ1 = dictionary_get_occ(dic1,i);
	    float occ2 = dictionary_get_occ(dic2,i);

	    v1 = buffer1[j].val;
	    v2 = buffer2[j].val;


	    tmp = ((buffer1[j].val*occ1*size1 +
		    buffer2[j].val*occ2*size2) /
		   (occ1*size1 + occ2*size2));
	    buffer1[j].val = tmp;

	    /* fprintf(stderr, "(%f x %f x %d + %f x %f x %d)/(%f x %d + %f x %d)= %f\n", 
	       v1, occ1, size1,
	       v2, occ2, size2,
	       occ1, size1,
	       occ2, size2,
	       buffer1[j].val); */

	    buffer1[j].id = buffer1[j].id?buffer1[j].id:buffer2[j].id;
	}

	/* Buffers somados. */
	/* Ordenar */
	qsort(buffer1, MAXENTRY*2, sizeof(DicPair), &cmp);

	/* preencher o novo dicionrio */
	for (j = 0; j < MAXENTRY; j++) {
	    dictionary_set_id(new, i, j, buffer1[j].id);
	    if (buffer1[j].val > 1) {
		fprintf(stderr, "Aqui vai um... %f\n", buffer1[j].val);
	    }
	    dictionary_set_val(new, i, j, buffer1[j].val);
	}
	
	count = dictionary_get_occ(dic1, i) + dictionary_get_occ(dic2, i);
	new = dictionary_set_occ(new, i, count);
    }

    return new;
}

double dictionary_sentence_similarity(Dictionary *dic,
				      guint32 *s1, int s1size,
				      guint32 *s2, int s2size) {
    double val = 0.0;
    int i, j, k, done;

    /* # remove some specific words if the stopwrds option is present;
       if ($conf->{stopwrds}) {
       my %tmp;
       @tmp{@{$conf->{stopwrds}}} = @{$conf->{stopwrds}};
       @wrds1 = grep { !exists($tmp{$_}) } @wrds1 if ($conf->{stopwrds});
       }
    */

    /* # number of words to be checked */
    /* $s1size = $conf->{size} if exists($conf->{size}); */

    
    for (i = 0; i < s1size; i++) {
	done = 0;
	if (s1[i] < dic->size) {
	    for (j = 0; j < MAXENTRY && !done; j++) {
		guint32 id = dictionary_get_id(dic, s1[i], j);
		for (k = 0; k < s2size && !done; k++) {
		    if (s2[k] == id) {
			float v = dictionary_get_val(dic, s1[i], j);
			val += ((double)1/s1size)*v;
			done = 1;
		    }
		}
	    }
	}
    }

    return val;

}
