/*****************************************************************************
 * TWENTE WORD ALIGNMENT TOOLS
 *
 * Copyright (C) 1998 Djoerd Hiemstra
 * University of Twente, PO Box 217, 7500 AE, Enschede, The Netherlands
 * email: hiemstra@cs.utwente.nl
 * 
 * - isolatin.c  : library file for text handling
 * 
 * This program is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by the Free 
 * Software Foundation; either version 2 of the License, or (at your option) 
 * any later version. The program is distributed in the hope that it will be 
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General 
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc., 59 
 * Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include "isolatin.h"


/* sd and hd are soft and hard delimiters, respectively '#' and '@'
   that are somewhat hard-coded in 'InWord', that is, they must be 
   'words' also: ASCII characters 35 and 64
*/


int InFile(char ch)
{
    return (ch >=33 && ch <= 126);
}

int InWord(char ch)
{
    /*  int i;
	i = (int) ch;
	if (i < 0) i += 256;
	return ((i >= 64 && i <= 90) || (i >= 97 && i <= 122) ||
	(i >= 192 && i<=254) || i == 35 || i == 39 || i == 60
	|| i == 62);
    */
    /* tokenising now by perl script */
    return (ch != ' ' && ch != '\n' && ch != '\t'); 
}

/* 
 * text: text to search
 * funct: function to check if a char is from a word
 *
 * returns: pointer to the beginning of the word on the text.
 */
char *FirstTextWord(char *text, int (*funct)(char))
    /* Searches begin of first word, skipping leading spaces, etc. */
{
    while (*text != '\0' && !(*funct)(*text))
	text++;
    if (*text == '\0') return NULL;
    else return text;
}

/*
 * text: text to search
 * funct: function to check if a char is from a word
 *
 * returns: pointer to the beginning of the word on the text
 */
char *NextTextWord(char *text, int (*funct)(char))
    /* Searches begin of next word, marking the current word with a \0 character */
{
    char* bow;
    bow = text;
    /* we are in the beginning of a word. Find its end! */
    while (*text != '\0' && (*funct)(*text))
	text++;
    /* if we end the buffer, return NULL */
    if (*text == '\0') return NULL;
    /* Mark end of the current word */
    *text++ = '\0';

    /* Search for the beginning of the next word */
    while (*text != '\0' && !(*funct)(*text))
	text++;
    /* if we end the buffer, return NULL */
    if (*text == '\0') return NULL;
    else return text;
}

/* 
 * sen: pointer to a buffer where the resulting sentence will be returned (list of words)
 * text: pointer to a buffer where is the text to be searched
 * maxLen: maximum size of the string
 * sd: SoftDelimiter
 * hr: HardDelimiter
 * funct: function saying if a char is in a word, or not.
 */
unsigned short NextTextString(char **sen, char **text, unsigned short maxLen,
			      char sd, char hd, int (*funct)(char))
{
    char *word;
    unsigned short len = 0;

    if (*text != NULL) {
	word = FirstTextWord(*text, funct);
	while (word != NULL && *word != sd) {
	    if (len < maxLen) {
		if (*word != hd) sen[(len)++] = word;
	    } 
	    else { 
		len ++;                              /* DUMMY stat */
	    }
	    word = NextTextWord(word, funct);
	}
	if (word != NULL)
	    word = NextTextWord(word, funct);
	if (word != NULL && *word == hd)
	    word = NextTextWord(word, funct);
	*text = word;
    }
    return len;
}

/* 
 * sen:  pointer to buffer where sentence will be returned;
 * text: pointer to the text where to search;
 * maxLen: maximum size of the Sentence;
 * sd: SoftDelimiter
 * hd: HardDelimiter
 */
unsigned short NextTextSentence(char **sen, char **text, unsigned short maxLen, char sd, char hd)
{
    return NextTextString(sen, text, maxLen, sd, hd, InWord);
}

unsigned short NextTextFile(char **sen, char **text, unsigned short maxLen, char d)
{
    return NextTextString(sen, text, maxLen, d, d, InFile);
}

/* Reads all text from file to a text buffer */
char *ReadText(const char *filename)
{
    FILE *fd;
    long len;
    struct stat stat_buf;
    char *result;
    
    fd = fopen(filename, "rt");
    if(fd == NULL) return NULL;
    if(fstat(fileno(fd), &stat_buf) == -1) return NULL;
    len = stat_buf.st_size;
    result = (char *) malloc(len+1);
    if(result == NULL) return NULL;
    if (fread(result, sizeof(char), len, fd) != len)
	return NULL;
    if (fclose(fd)) return NULL;
    result[len] = '\0';
    return(result);
}
