/* -*- Mode: C; c-file-style: "stroustrup" -*- */

/**********************************************
 * SEE COPYING FILE FOR COPYRIGHT INFORMATION *
 **********************************************/

#include <stdlib.h>
#include <stdio.h>
#include "standard.h"
#include "words.h"
#include "dict.h"
#include "dictionary.h"


#define MINTOT 0.0f

void setEntry(Dictionary* dic, guint32 word, 
	      WordList *WL, WordLstNode **WA,
	      guint32 c[MAXENTRY], float *f, guint32 max, float total)
{
    guint32 j;
    float v;

    for(j = 0; j < max; j++) {
	v = f[j]/total;
	if (c[j] && v > 0.005f) {
	    dic = dictionary_set_id(dic, word, j, c[j]);
	    dic = dictionary_set_val(dic, word, j, v);
	}
    }
}

void saveDicts(char *f1, char* f2, struct cMat2 *M, 
	       WordList *WL1,     WordList *WL2,
	       WordLstNode **W1,  WordLstNode **W2)
{
    float f[MAXENTRY], total;
    guint32 r, N, c[MAXENTRY];
    guint32 index;
    WordLstNode *node;
    Dictionary* dic;
    
    N = word_list_size(WL1);

    dic = dictionary_new(N);

    index = 2;
    for (r = 2; r <= N; r++) {   
	total = GetRowMax2(M, index, c, f, MAXENTRY);
	if (total > (float) MINTOT) setEntry(dic, index, WL2, W2, c, f, MAXENTRY, total);
	node = word_list_get_full_by_id(W1, index);
	dic = dictionary_set_occ(dic, index, node->count);
	index++;
    }
    total = GetRowMax2(M, 1, c, f, MAXENTRY);
    setEntry(dic, 1, WL2, W2, c, f, MAXENTRY, total); /*  */

    if (!dictionary_save(dic, f1))
	report_error("Cannot save dictionary 2");

    dictionary_free(dic);


    /*  */

    /* SECOND DICTIONARY */

    N = word_list_size(WL2);

    dic = dictionary_new(N);

    index = 2;
    for(r = 2; r <= N; r++) {   
	total = GetColumnMax2(M, index, c, f, MAXENTRY);
	if (total > (float) MINTOT) setEntry(dic, index, WL1, W1, c, f, MAXENTRY, total);
	node = word_list_get_full_by_id(W2, index);
	dic = dictionary_set_occ(dic, index, node->count);
	index++;
    }   
    total = GetColumnMax2(M, 1, c, f, MAXENTRY);   
    setEntry(dic, 1, WL1, W1, c, f, MAXENTRY, total); /*  */

    if (!dictionary_save(dic, f2))
	report_error("Cannot save dictionary 2");

    dictionary_free(dic);
}

int main(int argc, char **argv)
{
    struct cMat2 Dictionary;
    WordList *Words1, *Words2;
    WordLstNode **W1, **W2;

    W1 = NULL;
    W2 = NULL;
    
    if (argc != 6)
	report_error("Usage: post dictfile wordsfile1 wordsfile2 out1 out2");
    
    printf("\n");
    printf("Loading %s...\n", argv[1]);
    if (LoadMatrix2(&Dictionary, argv[1])) 
	report_error("load dictionary");

    printf("Loading %s...\n", argv[2]);
    if (!(Words1 = word_list_load(argv[2], &W1)))
	report_error("load strings 1");

    printf("Loading %s...\n", argv[3]);
    if (!(Words2 = word_list_load(argv[3], &W2)))
	report_error("load strings 2");

    printf("Writing dictionaries %s,%s...\n\n", argv[4], argv[5]);
    saveDicts(argv[4], argv[5], &Dictionary, Words1, Words2, W1, W2);

    printf("Freeing data structures\n");
    FreeMatrix2(&Dictionary);
    word_list_free(Words1);
    word_list_free(Words2);
    
    return 0;
}

