#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "words.h"

#define MAXWORDLEN 99


char nopunct(char c)
{
    if (c > 0) return c;
    else {
        int i;
        i = (int) c + 256;
        switch (i) {
        case 192: case 193: case 194: case 195: case 196: case 197:
            return 'A'; 
        case 199:
            return 'C'; 
        case 200: case 201: case 202: case 203:
            return 'E'; 
        case 204: case 205: case 206: case 207:
            return 'I'; 
        case 209:
            return 'N'; 
        case 210: case 211: case 212: case 213: case 214: case 216:
            return 'O'; 
        case 217: case 218: case 219: case 220:
            return 'U'; 
        case 221:
            return 'Y'; 
        case 224: case 225: case 226: case 227: case 228: case 229:
            return 'a'; 
        case 231:
            return 'c'; 
        case 232: case 233: case 234: case 235:
            return 'e'; 
        case 236: case 237: case 238: case 239:
            return 'i'; 
        case 241:
            return 'n'; 
        case 242: case 243: case 244: case 245: case 246: case 248:
            return 'o'; 
        case 249: case 250: case 251: case 252:
            return 'u'; 
        case 253:
            return 'y'; 
        default:
            return c; 
        }
    }
}

char tolowerx(char c)
{
    if (c > 0) 
        return tolower(c);     
    else if (c < -34)
        return c + 32;
    else 
        return c;
}


int strcmp_sort(char *s1, char *s2)
{
    int result;
    do
        result = tolower(nopunct(*s1++)) - tolower(nopunct(*s2++));
    while (!result && *s1 && *s2);
    if (!result && *s1) return 1;
    if (!result && *s2) return -1;
    else return result;
}

int strcmpx(const char *s1,const char *s2)
{
    int result;
    do
        result = (*s1++) - (*s2++);
    while (!result && *s1 && *s2);
    if (*(s1-1) == '*' || *(s2-1) == '*') result = 0;
    if (!result && *s1) return 1;
    if (!result && *s2) return -1;
    else return result;
}

WordList* word_list_new(void) {
    WordList *list = g_new(WordList, 1);
    list->count = 1; 		/* FIRST IS THE NULL */
    list->tree = NULL;
    return list;
}

WordLstNode* word_list_add_word_(WordLstNode* list, char *string, guint32* rn) {
    int cmp;
    if (!list) {
        WordLstNode *cell = g_new(WordLstNode, 1);
        if (!cell) { *rn = 0; return list; }

        cell->string = string;
        cell->count = 2;        /* I would put 1... but original uses 2 */
        cell->left = NULL;
        cell->right = NULL;
        cell->id = *rn;
        return cell;
    } else {
        cmp = strcmpx(string, list->string);
        if (cmp < 0) {
            list->left = word_list_add_word_(list->left, string, rn);
        } else if (cmp > 0) {
            list->right = word_list_add_word_(list->right, string, rn);
        } else {
            *rn = list->id;
            list->count++;
        }
	return list;
    }
}

guint32 word_list_add_word(WordList* list, char *string) {
    guint32 register_number = list->count + 1;
    
    list->tree = word_list_add_word_(list->tree, string, &register_number);
    if (register_number == list->count+1) list->count++;

    return register_number;
}

guint32 word_list_size(WordList* list) {
    return list->count;
}

void word_save_(WordLstNode* tree, FILE *fd) {
    if (tree) {
        fwrite(&tree->id, sizeof(tree->id), 1, fd);
        fwrite(&tree->count, sizeof(tree->count), 1, fd);
        fwrite(tree->string, strlen(tree->string)+1, 1, fd);
        word_save_(tree->left, fd);
        word_save_(tree->right, fd);
    }
}

gboolean word_list_save(WordList* list, char* filename) {
    FILE *fd;

    fd = fopen(filename, "w");
    if (fd == NULL)
        return FALSE;
    else {
        fwrite(&list->count, sizeof(list->count), 1, fd);
        word_save_(list->tree, fd);
    }
    fclose(fd);
    return TRUE;
}


WordLstNode* word_list_add_full_(WordLstNode* tree, guint32 id,
                               guint32 count, char* string,
                               WordLstNode ***ptr) {
    int cmp;
    if (!tree) {
        WordLstNode *cell = g_new(WordLstNode, 1);
        if (!cell) return tree;

        if (ptr && *ptr) {
            (*ptr)[id] = cell;
        }

        cell->string = string;
        cell->count = count;    /* I would put 1... but original uses 2 */
        cell->left = NULL;
        cell->right = NULL;
        cell->id = id;
        
        return cell;

    } else {
        cmp = strcmpx(string, tree->string);
        if (cmp < 0) {
            tree->left = word_list_add_full_(tree->left, id, count, string, ptr);
            return tree;
        } else if (cmp > 0) {
            tree->right = word_list_add_full_(tree->right, id, count, string, ptr);
            return tree;
        } else {
            return tree;
        }
    }
}

WordList* word_list_add_full(WordList* list, guint32 id,
                               guint32 count, const char* string, WordLstNode*** ptr) {
    char *str = strdup(string);
    list->tree = word_list_add_full_(list->tree, id, count, str, ptr);
    list->count++;              /* we hope this is not called for two equal strings */
    return list;
}


/* 
   ptr is used to construct a direct access array by id.
   use NULL to do not create it!
*/
WordList* word_list_load(const char *filename, WordLstNode*** ptr) {
    FILE *fd;
    int i;
    guint32 count;
    guint32 id, wc;
    char buffer[MAXWORDLEN+1];
    char ch;
    
    WordList *tree;
    tree = word_list_new();
    
    fd = fopen(filename, "rb");
    
    if (fd == NULL)
        return NULL;
    else {
        if (!fread(&wc, sizeof(wc), 1, fd)) return NULL;
        
        if (ptr) {
            *ptr = g_new0(WordLstNode*, wc+1);
            if (!*ptr) return NULL;
        }
        
        while(!feof(fd)) {
            fread(&id, sizeof(id), 1, fd);
            if (!feof(fd)) {
                fread(&count, sizeof(count), 1, fd);
                i = 0;
                while ((ch = fgetc(fd))) {
                    buffer[i++] = ch;
                }
                buffer[i] = '\0';
                word_list_add_full(tree, id, count, buffer, ptr);
            }
        }
        fclose(fd);
        return tree;
    }
}


void PrintWords(WordLstNode *Words)
{
    if (Words) {
        PrintWords(Words->left);
        printf(" '%s'\n", Words->string);
        PrintWords(Words->right);
    }
}

void word_list_print(char* title, WordList *lst) {
    printf("== %s ==\n", title);
    PrintWords(lst->tree);
}

void word_list_free_(WordLstNode *l) {
    if (l) {
        word_list_free_(l->left);
        word_list_free_(l->right);
        g_free(l);
    }
}

void word_list_free(WordList *lst) {
    word_list_free_(lst->tree);
    g_free(lst);
}

char* word_list_get_by_id(WordLstNode** ptr, guint32 index) {
    WordLstNode *cell;

    cell = ptr[index];
    if (cell) {
        return cell->string;
    } else {
        return NULL;
    }
}

WordLstNode *word_list_get_full_by_id(WordLstNode  **ptr,
				      guint32        index) {
    WordLstNode *cell;

    cell = ptr[index];
    if (cell) {
        return cell;
    } else {
        return NULL;
    }
}


guint32 get_id_(WordLstNode *tree, const char *string) {
    if (tree) {
        int cmp = strcmpx(string, tree->string);
        if (cmp<0) {
            return get_id_(tree->left, string);
        } else if (cmp>0) {
            return get_id_(tree->right, string);
        } else {
            return tree->id;
        }
    } else {
        return 0;
    }
}


guint32 word_list_get_id(WordList* list, const char *string) {
    return  get_id_(list->tree, string);
}


